package com.nokia.domainb.capability;

import com.alibaba.fastjson2.JSON;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.nokia.domainb.util.AESUtil;
import de.siegmar.fastcsv.reader.CsvReader;
import de.siegmar.fastcsv.reader.CsvRow;
import de.siegmar.fastcsv.writer.CsvWriter;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.util.StopWatch;

import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
class UsercomplaintQuerysTest {
    public static final String TEST_DIR = "test/";

    // 单例测试
    @Test
    void testQuery() throws Exception {
//        String username = "n1test004";
        String username = "test";
        String token = AESUtil.getToken(username);
        // 单例测试时修改参数
        String mobile = "15369158994";
        String destAddr = "";
        // 所有接收101 指定接收102 所有发送 103 指定发送104
        String flowNo = "103";
//        String flowNo = "103";
//        String beginTime = "20221029001228";
//        String endTime = "20221105001228";
        LocalDateTime endTimeLocalDateTime = LocalDateTime.now();
        String endTime = AESUtil.getTimeString(endTimeLocalDateTime);
        String beginTime = AESUtil.getTimeString(endTimeLocalDateTime.minusDays(7));
//        String complaintType = "2";
        String complaintType = "1";
        // 可以统一传空
        String isMobile = "";
        UsercomplaintQuerys usercomplaintQuerys = new UsercomplaintQuerys();
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        String result = usercomplaintQuerys.queryDirect(mobile, destAddr, complaintType, isMobile, flowNo, beginTime,
                endTime, token, false);
//        COMPConnectionContext result = usercomplaintQuerys.query(mobile, destAddr, complaintType, isMobile, flowNo,
//                beginTime, endTime, token, false);
        stopWatch.stop();
        Map<String, String> req = new HashMap<>();
        req.put("mobile", mobile);
        req.put("destAddr", destAddr);
        req.put("complaintType", complaintType);
        req.put("isMobile", isMobile);
        req.put("flowNo", flowNo);
        req.put("beginTime", beginTime);
        req.put("endTime", endTime);
        req.put("token", token);
        Map<Object, Object> map = new HashMap<>();
        map.put("req", req);
        Object res = JSON.parse(result);
//        Object res = JSON.parse(result.getResponseContext());
        map.put("res", res);
        log.info("入参：{}", JSON.toJSONString(req));
        log.info("{} ms | 结果：{}", stopWatch.getTotalTimeMillis(), res);
//        log.info("{} ms | 结果：{}", stopWatch.getTotalTimeMillis(), result.getResponseContext());
        log.info("map: {}", JSON.toJSONString(map));
    }

    /**
     * 提取ete_complaint_query_log_tab的query_param
     */
    @Test
    void testGetQueryParam() {
        String dataPath = TEST_DIR + "ete_complaint_query_log_tab.csv";
        String resultPath = TEST_DIR + "data.csv";
        try (CsvReader reader = CsvReader.builder().build(Paths.get(dataPath), StandardCharsets.UTF_8);
             OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(Paths.get(resultPath)),
                     StandardCharsets.UTF_8);
             CsvWriter writer = CsvWriter.builder().build(osw);
        ) {
            Map<String, List<String>> map = new HashMap<>();
            for (CsvRow row : reader) {
                if (row.getOriginalLineNumber() == 1) {
                    continue;
                }
                String queryParam = row.getField(3);
                JsonObject jsonObject = JsonParser.parseString(queryParam).getAsJsonObject();
                String mobile = jsonObject.get("mobile").getAsString();
                String flowNo = jsonObject.get("flowNo").getAsString();
                String complaintType = jsonObject.get("complaintType").getAsString();
                List<String> list = new ArrayList<>();
                list.add(mobile);
                list.add(flowNo);
                list.add(complaintType);
                map.putIfAbsent(mobile, list);
            }
            for (List<String> value : map.values()) {
                writer.writeRow(value);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    @Test
    void testRepeat() {
        // 数据文件路径
        String dataPath = TEST_DIR + "data.csv";
        String resultPath = TEST_DIR + "result.csv";
        try (CsvReader reader = CsvReader.builder().build(Paths.get(dataPath), StandardCharsets.UTF_8);
             OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(Paths.get(resultPath)),
                     "gbk");
             CsvWriter writer = CsvWriter.builder().build(osw)) {
            writer.writeRow("号码", "时间", "耗时(s)", "耗时(ms)", "入参", "结果");
            List<Map> list = new ArrayList<>();
            for (CsvRow row : reader) {
                Map map = JSON.parseObject(row.getField(3), Map.class);
                log.info("map: {}", JSON.toJSONString(map));
                list.add(map);
            }
            for (int i = 0; i < 10; i++) {
                log.info("{}>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", i);
                for (Map req : list) {
//                    String username = "test";
//                    String token = AESUtil.getToken(username);
//                    req.put("token", token);
                    String res;
                    UsercomplaintQuerys usercomplaintQuerys = new UsercomplaintQuerys();
                    LocalDateTime now = LocalDateTime.now();
                    StopWatch stopWatch = new StopWatch();
                    stopWatch.start();
                    try {
                        res = usercomplaintQuerys.queryDirect(req, false);
                    } catch (Exception e) {
                        log.error(e.getMessage(), e);
                        res = e.getMessage();
                    }
                    stopWatch.stop();
                    List<String> data = new ArrayList<>();
                    data.add(req.get("mobile").toString());
                    data.add(now.toString());
                    data.add(String.valueOf((long) Math.ceil(stopWatch.getTotalTimeSeconds())));
                    data.add(String.valueOf(stopWatch.getTotalTimeMillis()));
                    data.add(JSON.toJSONString(req));
                    data.add(res);
                    writer.writeRow(data);
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }
}
