package com.nokia.domainb.capability.config;

import lombok.extern.slf4j.Slf4j;
import okhttp3.ConnectionPool;
import okhttp3.OkHttpClient;
import org.springframework.boot.test.context.TestConfiguration;
import org.springframework.context.annotation.Bean;
import org.springframework.http.client.OkHttp3ClientHttpRequestFactory;
import org.springframework.scheduling.concurrent.CustomizableThreadFactory;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.web.client.RestTemplate;

import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

@Slf4j
@TestConfiguration
public class TestConfig {
    public static final int qps = 2;
    public static final int corePoolSize = 100;
    public static final int maximumPoolSize = corePoolSize;
    public static LinkedBlockingQueue<List<String>> queue = new LinkedBlockingQueue<>();

    @Bean
    public RestTemplate template() {
        return new RestTemplate(new OkHttp3ClientHttpRequestFactory(new OkHttpClient().newBuilder()
                .connectionPool(new ConnectionPool(1000, 10, TimeUnit.MINUTES)).build()));
    }

    @Bean
    public ThreadPoolExecutor requestExecutor() {
        ThreadPoolExecutor t = new ThreadPoolExecutor(corePoolSize, maximumPoolSize,
                0L, TimeUnit.MILLISECONDS,
                new LinkedBlockingQueue<>(), new CustomizableThreadFactory("request-"));
        t.prestartAllCoreThreads();
        return t;
    }

    @Bean
    public ThreadPoolExecutor offerExecutor() {
        ThreadPoolExecutor t = new ThreadPoolExecutor(corePoolSize, maximumPoolSize,
                0L, TimeUnit.MILLISECONDS,
                new LinkedBlockingQueue<>(), new CustomizableThreadFactory("offer-"));
        t.prestartAllCoreThreads();
        return t;
    }

    @Bean
    public ThreadPoolTaskScheduler scheduler() {
        ThreadPoolTaskScheduler scheduler = new ThreadPoolTaskScheduler();
        scheduler.setThreadNamePrefix("scheduler-");
        scheduler.initialize();
        return scheduler;
    }
}
