package com.nokia.domainb.capability.sdk;

import cn.chinaunicom.open.nlgxptconnection.COMPConnectionContext;
import com.alibaba.fastjson2.JSON;
import com.google.common.collect.Lists;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.nokia.domainb.capability.QryRouteAll;
import com.nokia.domainb.capability.UsercomplaintQuerys;
import com.nokia.domainb.capability.config.TestConfig;
import com.nokia.domainb.util.AESUtil;
import de.siegmar.fastcsv.reader.CsvReader;
import de.siegmar.fastcsv.reader.CsvRow;
import de.siegmar.fastcsv.writer.CsvWriter;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Import;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.util.StopWatch;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.Instant;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Slf4j
@SpringBootTest
@Import(TestConfig.class)
class SdkConcurrentTest {
    /**
     * 测试文件夹
     */
    public static final String TEST_DIR = "test/";

    @Autowired
    private ThreadPoolExecutor requestExecutor;
    @Autowired
    private ThreadPoolExecutor offerExecutor;
    @Autowired
    private ThreadPoolTaskScheduler scheduler;

    @Test
    void testQryRouteAll() {
        // 数据文件路径
        String dataPath = TEST_DIR + "data.csv";
        // 结果保存的文件路径
        String resultPath = TEST_DIR + "result.csv";
        try(Stream<String> lines = Files.lines(Paths.get(dataPath))) {
            // 去重
            List<String> list = lines.distinct().collect(Collectors.toList());
            // 切分数据，每秒 qps 个请求
            List<List<String>> parts = Lists.partition(list, TestConfig.qps);
            // 任务执行时间
            Instant startTime = Instant.now();
            QryRouteAll qryRouteAll = new QryRouteAll();
            for (List<String> p : parts) {
                startTime = startTime.plusSeconds(1);
                // 添加定时任务
                scheduler.schedule(() -> {
                    // 并发请求
                    CompletableFuture.allOf(p.stream().map(t ->
                            CompletableFuture.runAsync(() -> {
                                String provinceCode = "";
                                String eparchyCode = "";
                                String result;
                                LocalDateTime now = LocalDateTime.now();
                                StopWatch stopWatch = new StopWatch();
                                stopWatch.start();
                                try {
                                    result = qryRouteAll.query(t);
                                    JsonObject jsonObject = JsonParser.parseString(result).getAsJsonObject();
                                    JsonObject rsp = jsonObject.getAsJsonObject("UNI_BSS_BODY")
                                            .getAsJsonObject("QRYROUTEALL_RSP")
                                            .getAsJsonObject("RSP");
                                    JsonObject data = rsp.getAsJsonArray("DATA").get(0).getAsJsonObject();
                                    JsonElement provinceCodeElement = data.get("PROVINCE_CODE");
                                    if (provinceCodeElement != null) {
                                        provinceCode = data.get("PROVINCE_CODE").getAsString();
                                    }

                                    JsonElement eparchyCodeElement = data.get("EPARCHY_CODE");
                                    if (eparchyCodeElement != null) {
                                        eparchyCode = eparchyCodeElement.getAsString();
                                    }
                                } catch (Exception e) {
                                    log.error(e.getMessage(), e);
                                    result = e.getMessage();
                                }
                                stopWatch.stop();
                                List<String> data = new ArrayList<>();
                                data.add(t);
                                data.add(provinceCode);
                                data.add(eparchyCode);
                                data.add(now.toString());
                                data.add(String.valueOf((long) Math.ceil(stopWatch.getTotalTimeSeconds())));
                                data.add(String.valueOf(stopWatch.getTotalTimeMillis()));
                                data.add(result);
                                // 结果入队列
                                CompletableFuture.runAsync(() -> TestConfig.queue.offer(data), offerExecutor);
                            }, requestExecutor)).toArray(CompletableFuture[]::new));
                }, startTime);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        writeFile(resultPath);
    }

    @Test
    void testUserComplaintQuery() {
        // 结果保存的文件夹路径
        String resultDirPath = TEST_DIR + "result/userComplaintQuery/";
        // 创建文件夹
        try {
            Files.createDirectories(Paths.get(resultDirPath));
        } catch (IOException e) {
            log.error("创建文件夹失败: {}", e.getMessage(), e);
            return;
        }
        // 查询时间范围
        long day = 5;
        // 结果保存文件
        String resultPath = resultDirPath + "短信用户投诉分析查询接口" + day + "天范围" + TestConfig.qps + "个线程1000条数据测试结果.csv";
        userComplaintQuery(day, resultPath);
//        day = 6;
//        resultPath = resultDirPath + "短信用户投诉分析查询接口" + day + "天范围" + TestConfig.qps + "个线程1000条数据测试结果.csv";
//        userComplaintQuery(day, resultPath);
//        day = 7;
//        resultPath = resultDirPath + "短信用户投诉分析查询接口" + day + "天范围" + TestConfig.qps + "个线程1000条数据测试结果.csv";
//        userComplaintQuery(day, resultPath);
    }

    public void userComplaintQuery(long day, String resultPath) {
        // 数据文件路径
        String dataPath = TEST_DIR + "data.csv";
        String username = "test";
        String token;
        try {
            token = AESUtil.getToken(username);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return;
        }
        String destAddr = "";
        // 可以统一传空
        String isMobile = "";
        LocalDateTime endTimeLocalDateTime = LocalDateTime.now();
        String endTime = AESUtil.getTimeString(endTimeLocalDateTime);
        String beginTime = AESUtil.getTimeString(endTimeLocalDateTime.minusDays(day));
        List<Map<String, String>> list = new ArrayList<>();
        try (CsvReader reader = CsvReader.builder().build(Paths.get(dataPath), StandardCharsets.UTF_8)) {
            for (CsvRow row : reader) {
                String mobile = row.getField(0);
                // 所有接收101 指定接收102 所有发送 103 指定发送104
                String flowNo = row.getField(1);
                String complaintType = row.getField(2);
                Map<String, String> reqMap = new HashMap<>();
                reqMap.put("mobile", mobile);
                reqMap.put("destAddr", destAddr);
                reqMap.put("complaintType", complaintType);
                reqMap.put("isMobile", isMobile);
                reqMap.put("flowNo", flowNo);
                reqMap.put("beginTime", beginTime);
                reqMap.put("endTime", endTime);
                reqMap.put("token", token);
                list.add(reqMap);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return;
        }
        UsercomplaintQuerys usercomplaintQuerys = new UsercomplaintQuerys();
        // 切分数据，每秒 qps 个请求
        List<List<Map<String, String>>> parts = Lists.partition(list, TestConfig.qps);
        // 任务执行时间
        Instant startTime = Instant.now();
        for (List<Map<String, String>> p : parts) {
            startTime = startTime.plusSeconds(1);
            // 添加定时任务
            scheduler.schedule(() -> {
                // 并发请求
                CompletableFuture.allOf(p.stream().map(t ->
                        CompletableFuture.runAsync(() -> {
                            Object res;
                            LocalDateTime now = LocalDateTime.now();
                            StopWatch stopWatch = new StopWatch();
                            stopWatch.start();
                            try {
                                COMPConnectionContext r = usercomplaintQuerys.query(t, false);
                                res = r.getResponseContext();
                            } catch (Exception e) {
                                log.error(e.getMessage(), e);
                                res = e.getMessage();
                            }
                            stopWatch.stop();
                            List<String> data = new ArrayList<>();
                            data.add(now.toString());
                            data.add(String.valueOf((long) Math.ceil(stopWatch.getTotalTimeSeconds())));
                            data.add(String.valueOf(stopWatch.getTotalTimeMillis()));
                            data.add(JSON.toJSONString(t));
                            data.add(JSON.toJSONString(res));
                            // 结果入队列
                            CompletableFuture.runAsync(() -> TestConfig.queue.offer(data), offerExecutor);
                        }, requestExecutor)).toArray(CompletableFuture[]::new));
            }, startTime);
        }
        writeFile(resultPath);
    }

    /**
     * 写文件
     *
     * @param path 写入文件路径
     */
    public void writeFile(String path) {
        ScheduledThreadPoolExecutor scheduledExecutor = scheduler.getScheduledThreadPoolExecutor();
        try (OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(Paths.get(path)),
                "gbk");
             CsvWriter csv = CsvWriter.builder().build(osw)) {
//            csv.writeRow("时间", "耗时(s)", "耗时(ms)", "入参", "结果");
            csv.writeRow("phone","provinceCode","eparchyCode","时间", "耗时(s)", "耗时(ms)", "结果");
            while (scheduler.getActiveCount() > 0
                    || !scheduledExecutor.getQueue().isEmpty()
                    || scheduledExecutor.getCompletedTaskCount() < scheduledExecutor.getTaskCount()
                    || requestExecutor.getActiveCount() > 0
                    || !requestExecutor.getQueue().isEmpty()
                    || requestExecutor.getCompletedTaskCount() < requestExecutor.getTaskCount()
                    || offerExecutor.getActiveCount() > 0
                    || !offerExecutor.getQueue().isEmpty()
                    || offerExecutor.getCompletedTaskCount() < offerExecutor.getTaskCount()
                    || !TestConfig.queue.isEmpty()) {
                List<String> t = TestConfig.queue.take();
                csv.writeRow(t);
                log.info("schedulerCompletedTaskCount: {}, schedulerTaskCount: {}, schedulerQueueSize: {}, schedulerActiveCount: {}, requestCompletedTaskCount: {}, requestTaskCount: {}, requestQueueSize: {}, requestActiveCount: {}, offerCompletedTaskCount: {}, offerTaskCount: {}, offerExecutorQueueSize: {}, OfferActiveCount: {}",
                        scheduledExecutor.getCompletedTaskCount(),
                        scheduledExecutor.getTaskCount(),
                        scheduledExecutor.getQueue().size(),
                        scheduler.getActiveCount(),
                        requestExecutor.getCompletedTaskCount(),
                        requestExecutor.getTaskCount(),
                        requestExecutor.getQueue().size(),
                        requestExecutor.getActiveCount(),
                        offerExecutor.getCompletedTaskCount(),
                        offerExecutor.getTaskCount(),
                        offerExecutor.getQueue().size(),
                        offerExecutor.getActiveCount());
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }
}
